import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.rpc.ServiceException;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.sforce.soap.enterprise.Error;
import com.sforce.soap.enterprise.LoginResult;
import com.sforce.soap.enterprise.SessionHeader;
import com.sforce.soap.enterprise.SforceServiceLocator;
import com.sforce.soap.enterprise.SoapBindingStub;
import com.sforce.soap.enterprise.UpsertResult;
import com.sforce.soap.enterprise.fault.ApiFault;
import com.sforce.soap.enterprise.fault.LoginFault;
import com.sforce.soap.enterprise.sobject.Resource__c;
import com.sforce.soap.enterprise.sobject.SObject;

public class IntegrationDemo {
  SoapBindingStub binding;
  String sessionId;
  
  public void login(String user, String pass, String securityToken) {
    try {
      SforceServiceLocator locator = new SforceServiceLocator();
      binding = (SoapBindingStub)locator.getSoap();
    } catch (ServiceException e) {
      e.printStackTrace();
    }
  
    try {
      LoginResult loginResult = binding.login(user, pass + securityToken);
      binding._setProperty(SoapBindingStub.ENDPOINT_ADDRESS_PROPERTY,
              "https://foo.com/" + loginResult.getServerUrl());
      SessionHeader sh = new SessionHeader();
      sessionId = loginResult.getSessionId();
      sh.setSessionId(sessionId);
      binding.setHeader(new SforceServiceLocator().getServiceName()
        .getNamespaceURI(), "SessionHeader", sh);
    } catch (LoginFault e) {
      System.out.println("ExceptionCode: " + e.getExceptionCode());
    } catch (ApiFault e) {
      e.printStackTrace();
    } catch (RemoteException e) {
      e.printStackTrace();
    }
  }

  public void processImportFile(String jsonFile) {
    List<SObject> changes = new ArrayList<SObject>();
    try {
      String json = readFileAsString(jsonFile);
      JSONArray array = new JSONArray(json);
      for (int i=0; i<array.length(); i++) {
        changes.add(importResource(array.getJSONObject(i)));
      }
      if (changes.size() > 0) {
        UpsertResult[] results = binding.upsert("Resource_ID__c", changes.toArray(new SObject[changes.size()]));
        int line = 0;
        for (UpsertResult result : results) {
          System.out.print(line + ": ");
          if (!result.isSuccess()) {
            for (Error e : result.getErrors()) {
              System.out.println(e.getStatusCode() + ": " + e.getMessage());
            }
          } else {
            System.out.println("success");
          }
          line++;
        }
      }
    } catch (Throwable t) {
      t.printStackTrace();
    }
  }

  private Resource__c importResource(JSONObject rec)
    throws JSONException {
    Resource__c result = new Resource__c();
    result.setResource_ID__c(Double.valueOf(rec.getInt("ResourceID")));
    result.setActive__c(rec.getBoolean("Active"));
    return result;
  }
  
  private static String readFileAsString(String filePath)
    throws IOException {
    StringBuffer fileData = new StringBuffer(1000);
    BufferedReader reader = new BufferedReader(
            new FileReader(filePath));
    char[] buf = new char[2048];
    int numRead = 0;
    while((numRead = reader.read(buf)) != -1){
        fileData.append(buf, 0, numRead);
    }
    reader.close();
    return fileData.toString();
  }

  public static void main(String[] args) {
    IntegrationDemo demo = new IntegrationDemo();
    demo.login("paigeturner.dl@gmail.com", "pass8word", "WFEpxI9k8WYvWo5WXNzKAPOeq");
    demo.processImportFile("import.json");
  }
}
