import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.rpc.ServiceException;

import com.sforce.soap._2006._04.metadata.AsyncResult;
import com.sforce.soap._2006._04.metadata.CustomField;
import com.sforce.soap._2006._04.metadata.CustomObject;
import com.sforce.soap._2006._04.metadata.DeploymentStatus;
import com.sforce.soap._2006._04.metadata.FieldType;
import com.sforce.soap._2006._04.metadata.Metadata;
import com.sforce.soap._2006._04.metadata.MetadataBindingStub;
import com.sforce.soap._2006._04.metadata.MetadataServiceLocator;
import com.sforce.soap._2006._04.metadata.SharingModel;
import com.sforce.soap.enterprise.GetUserInfoResult;
import com.sforce.soap.enterprise.LoginResult;
import com.sforce.soap.enterprise.QueryResult;
import com.sforce.soap.enterprise.SaveResult;
import com.sforce.soap.enterprise.SessionHeader;
import com.sforce.soap.enterprise.SforceServiceLocator;
import com.sforce.soap.enterprise.SoapBindingStub;
import com.sforce.soap.enterprise.fault.ApiFault;
import com.sforce.soap.enterprise.fault.InvalidFieldFault;
import com.sforce.soap.enterprise.fault.InvalidIdFault;
import com.sforce.soap.enterprise.fault.InvalidSObjectFault;
import com.sforce.soap.enterprise.fault.LoginFault;
import com.sforce.soap.enterprise.fault.UnexpectedErrorFault;
import com.sforce.soap.enterprise.sobject.Contact;
import com.sforce.soap.enterprise.sobject.Proj__c;
import com.sforce.soap.enterprise.sobject.Resource__c;
import com.sforce.soap.enterprise.sobject.SObject;
import com.sforce.soap.schemas._class.Custom.CustomBindingStub;
import com.sforce.soap.schemas._class.Custom.CustomServiceLocator;

public class WSDemo {
  SoapBindingStub binding;
  String sessionId;
  
	public void login(String user, String pass, String securityToken) {
	  try {
    	SforceServiceLocator locator = new SforceServiceLocator();
      binding = (SoapBindingStub)locator.getSoap();
    } catch (ServiceException e) {
      e.printStackTrace();
    }
	
    try {
	    LoginResult loginResult = binding.login(user, pass + securityToken);
	    binding._setProperty(SoapBindingStub.ENDPOINT_ADDRESS_PROPERTY,
	            loginResult.getServerUrl());
	    SessionHeader sh = new SessionHeader();
	    sessionId = loginResult.getSessionId();
	    sh.setSessionId(sessionId);
	    binding.setHeader(new SforceServiceLocator().getServiceName()
        .getNamespaceURI(), "SessionHeader", sh);
		} catch (LoginFault e) {
			System.out.println("ExceptionCode: " + e.getExceptionCode());
		} catch (ApiFault e) {
		  e.printStackTrace();
		} catch (RemoteException e) {
		  e.printStackTrace();
		}
	}

	public String getUserName() {
	  try {
      GetUserInfoResult userInfo = binding.getUserInfo();
      return userInfo.getUserFullName();
    } catch (UnexpectedErrorFault e) {
      e.printStackTrace();
    } catch (RemoteException e) {
      e.printStackTrace();
    }
    return null;
	}

	public String createProject(String name) {
    CustomServiceLocator locator = new CustomServiceLocator();
    try {
      CustomBindingStub serviceBinding = (CustomBindingStub)locator.getCustom();
      SessionHeader sh = new SessionHeader();
      sh.setSessionId(sessionId);
      serviceBinding.setHeader(locator.getServiceName()
        .getNamespaceURI(), "SessionHeader", sh);
      return serviceBinding.createProject(name);
    } catch (ServiceException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (RemoteException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    return null;
	}
	
	public void createObject(String name, String pluralName) {
	  MetadataServiceLocator locator = new MetadataServiceLocator();
    try {
      MetadataBindingStub mdBinding = (MetadataBindingStub)locator.getMetadata();
      SessionHeader sh = new SessionHeader();
      sh.setSessionId(sessionId);
      mdBinding.setHeader(locator.getServiceName()
        .getNamespaceURI(), "SessionHeader", sh);
      CustomObject obj = new CustomObject();

      obj.setFullName(name + "__c");
      obj.setLabel(name);
      obj.setPluralLabel(pluralName);
      obj.setDeploymentStatus(DeploymentStatus.Deployed);
      obj.setSharingModel(SharingModel.ReadWrite);      
      
      CustomField nameField = new CustomField();
      nameField.setType(FieldType.AutoNumber);
      nameField.setLabel("Name");
      obj.setNameField(nameField);

      AsyncResult[] result = mdBinding.create(new Metadata[] { obj });
      if (result == null) {
        System.out.println("create failed");
        return;
      }
      
      boolean done = false;
      AsyncResult[] status = null;
      long waitTime = 1000;
      while (!done) {
        status = mdBinding.checkStatus(new String[] { result[0].getId() } );
        if (status != null) {
          done = status[0].isDone();
          if (status[0].getStatusCode() != null ) {
              System.out.println("Error: " +
                status[0].getStatusCode() + ": " + status[0].getMessage());
          }
          Thread.sleep(waitTime);
          waitTime *= 2;
          System.out.println("Current state: " + status[0].getState());
        }
    }
    
    System.out.println("Created object: " +
            status[0].getId());
    } catch (Throwable t) {
      t.printStackTrace();
    }
	}
	
	public List<Proj__c> queryProjects() {
	  List<Proj__c> projects = new ArrayList<Proj__c>();
	  try {
      QueryResult qr = binding.query("select Id, Name from Proj__c order by Name");
      boolean done = false;
      if (qr.getSize() > 0) {
        while (!done) {
        SObject[] records = qr.getRecords();
        if (records != null) {
          for (SObject record : records) {
            projects.add((Proj__c)record);
          }
          if (qr.isDone()) {
            done = true;
          } else {
            qr = binding.queryMore(qr.getQueryLocator());
          }
        }
        }
      }
	  } catch (Throwable t) {
	    t.printStackTrace();
	  }
	  return projects;
	}
	
	public void createResource(String firstName, String lastName) {
	  String newResourceId = null;
	  try {
  	  Contact contact = new Contact();
      contact.setFirstName(firstName);
      contact.setLastName(lastName);
      SaveResult[] result = binding.create(new SObject[] { contact });
      if (result != null && result.length == 1) {
        if (result[0].isSuccess()) {
          Resource__c resource = new Resource__c();
          resource.setActive__c(true);
          resource.setName(firstName + " " + lastName);
          resource.setResource_ID__c(123d);
          resource.setContact__c(result[0].getId());
          SaveResult[] result2 = binding.create(new SObject[] { resource });
          if (result2 != null && result2.length == 1) {
            if (result2[0].isSuccess()) {
              newResourceId = result2[0].getId();
            } else {
              System.out.println("Failed to create resource: " + result2[0].getErrors(0).getMessage());
            }
          }
        } else {
          System.out.println("Failed to create contact: " + result[0].getErrors(0).getMessage());
        }
      }
	  } catch (Throwable t) {
	    t.printStackTrace();
	  }
	  System.out.println(newResourceId);
	}

	public static void main(String[] args) {
		WSDemo demo = new WSDemo();
		demo.login("paigeturner.dl@gmail.com", "pass8word", "WFEpxI9k8WYvWo5WXNzKAPOeq");
		System.out.println(demo.getUserName());
	//	demo.queryProjects();
//		demo.query()
//    System.out.println(demo.createProject("test!"));
//		demo.createObject("Foo", "Foos");
		demo.createResource("Foo", "Bar");
	}
}
