import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import javax.xml.namespace.QName;

import org.joda.time.DateTime;
import org.opensaml.Configuration;
import org.opensaml.DefaultBootstrap;
import org.opensaml.common.IdentifierGenerator;
import org.opensaml.common.SAMLVersion;
import org.opensaml.common.impl.SecureRandomIdentifierGenerator;
import org.opensaml.saml2.core.Assertion;
import org.opensaml.saml2.core.Audience;
import org.opensaml.saml2.core.AudienceRestriction;
import org.opensaml.saml2.core.AuthnContext;
import org.opensaml.saml2.core.AuthnStatement;
import org.opensaml.saml2.core.Conditions;
import org.opensaml.saml2.core.Issuer;
import org.opensaml.saml2.core.NameID;
import org.opensaml.saml2.core.Response;
import org.opensaml.saml2.core.Status;
import org.opensaml.saml2.core.StatusCode;
import org.opensaml.saml2.core.Subject;
import org.opensaml.saml2.core.SubjectConfirmation;
import org.opensaml.saml2.core.SubjectConfirmationData;
import org.opensaml.xml.ConfigurationException;
import org.opensaml.xml.XMLObject;
import org.opensaml.xml.XMLObjectBuilder;
import org.opensaml.xml.XMLObjectBuilderFactory;
import org.opensaml.xml.io.Marshaller;
import org.opensaml.xml.io.MarshallerFactory;
import org.opensaml.xml.io.UnmarshallerFactory;
import org.opensaml.xml.parse.BasicParserPool;
import org.opensaml.xml.security.SecurityHelper;
import org.opensaml.xml.security.credential.BasicCredential;
import org.opensaml.xml.signature.Signature;
import org.opensaml.xml.signature.SignatureConstants;
import org.opensaml.xml.signature.Signer;
import org.opensaml.xml.util.XMLHelper;
import org.w3c.dom.Element;

import sun.misc.BASE64Encoder;

public class SAMLDemo {
  protected static MarshallerFactory marshallerFactory;
  protected static UnmarshallerFactory unmarshallerFactory;
  protected static XMLObjectBuilderFactory builderFactory;
  protected static BasicParserPool parser;

  public SAMLDemo() throws ConfigurationException {
    DefaultBootstrap.bootstrap();
    parser = new BasicParserPool();
    parser.setNamespaceAware(true);
    marshallerFactory = Configuration.getMarshallerFactory();
    unmarshallerFactory = Configuration.getUnmarshallerFactory();
    builderFactory = Configuration.getBuilderFactory();
  }

  public String createSAMLResponse() {
    try {
      BasicCredential credential = readCredentials("cert.crt", "pk.der");
      IdentifierGenerator idGenerator = new SecureRandomIdentifierGenerator();
      DateTime now = new DateTime();

      Subject subject = (Subject) buildXMLObject(Subject.DEFAULT_ELEMENT_NAME);
      NameID nameID = (NameID) buildXMLObject(NameID.DEFAULT_ELEMENT_NAME);
      nameID.setFormat(NameID.EMAIL);
      nameID.setValue("paigeturner.dl@gmail.com");
      subject.setNameID(nameID);
      SubjectConfirmation conf = (SubjectConfirmation) buildXMLObject(SubjectConfirmation.DEFAULT_ELEMENT_NAME);
      conf.setMethod("urn:oasis:names:tc:SAML:2.0:cm:bearer");
      SubjectConfirmationData data = (SubjectConfirmationData) buildXMLObject(SubjectConfirmationData.DEFAULT_ELEMENT_NAME);
      data.setRecipient("https://login.salesforce.com");
      data.setNotOnOrAfter(now);
      conf.setSubjectConfirmationData(data);
      subject.getSubjectConfirmations().add(conf);

      Assertion assertion = (Assertion) buildXMLObject(Assertion.DEFAULT_ELEMENT_NAME);
      assertion.setVersion(SAMLVersion.VERSION_20);
      assertion.setID(idGenerator.generateIdentifier());
      assertion.setIssueInstant(now);
      assertion.setSubject(subject);

      Conditions conditions = (Conditions) buildXMLObject(Conditions.DEFAULT_ELEMENT_NAME);
      conditions.setNotOnOrAfter(new DateTime());
      conditions.setNotBefore(new DateTime());
      AudienceRestriction restriction = (AudienceRestriction) buildXMLObject(AudienceRestriction.DEFAULT_ELEMENT_NAME);
      Audience audience = (Audience) buildXMLObject(Audience.DEFAULT_ELEMENT_NAME);
      audience.setAudienceURI("https://saml.salesforce.com");
      restriction.getAudiences().add(audience);
      conditions.getAudienceRestrictions().add(restriction);
      assertion.setConditions(conditions);

      Issuer issuer = (Issuer) buildXMLObject(Issuer.DEFAULT_ELEMENT_NAME);
      issuer.setValue("dev.appirio.net");
      assertion.setIssuer(issuer);

      AuthnStatement authnStmt = (AuthnStatement) buildXMLObject(AuthnStatement.DEFAULT_ELEMENT_NAME);
      authnStmt.setAuthnInstant(now);
      assertion.getAuthnStatements().add(authnStmt);

      AuthnContext authnContext = (AuthnContext) buildXMLObject(AuthnContext.DEFAULT_ELEMENT_NAME);
      authnStmt.setAuthnContext(authnContext);

      Response response = (Response) buildXMLObject(Response.DEFAULT_ELEMENT_NAME);
      response.setID(idGenerator.generateIdentifier());
      Issuer issuer2 = (Issuer) buildXMLObject(Issuer.DEFAULT_ELEMENT_NAME);
      issuer2.setValue("dev.appirio.net");
      response.setIssuer(issuer2);
      response.setIssueInstant(new DateTime());
      response.getAssertions().add(assertion);
      /**
       * Uncomment to set a logout URL.
       * 
       * AttributeStatement attrStmt = (AttributeStatement)
       * buildXMLObject(AttributeStatement.DEFAULT_ELEMENT_NAME); Attribute
       * logoutUrl = (Attribute) buildXMLObject(Attribute.DEFAULT_ELEMENT_NAME);
       * logoutUrl.setName("logoutURL");
       * logoutUrl.setNameFormat("urn:oasis:names:tc:SAML:2.0:attrname-format:uri"
       * );
       * 
       * XMLObjectBuilder stringBuilder =
       * builderFactory.getBuilder(XSString.TYPE_NAME); XSString logoutUrlValue
       * = null; logoutUrlValue = (XSString)
       * stringBuilder.buildObject(AttributeValue.DEFAULT_ELEMENT_NAME,
       * XSString.TYPE_NAME); logoutUrlValue.setValue("http://www.salesforce.com");
       * logoutUrl.getAttributeValues().add(logoutUrlValue);
       * attrStmt.getAttributes().add(logoutUrl);
       * assertion.getAttributeStatements().add(attrStmt);
       */
      Status status = (Status) buildXMLObject(Status.DEFAULT_ELEMENT_NAME);
      response.setStatus(status);
      StatusCode code = (StatusCode) buildXMLObject(StatusCode.DEFAULT_ELEMENT_NAME);
      code.setValue(StatusCode.SUCCESS_URI);
      status.setStatusCode(code);

      Signature signature = (Signature) buildXMLObject(Signature.DEFAULT_ELEMENT_NAME);
      signature.setSigningCredential(credential);
      signature
          .setCanonicalizationAlgorithm(SignatureConstants.ALGO_ID_C14N_EXCL_OMIT_COMMENTS);
      signature.setSignatureAlgorithm(SignatureConstants.ALGO_ID_SIGNATURE_RSA);
      response.setSignature(signature);

      Marshaller marshaller = marshallerFactory.getMarshaller(response);
      marshaller.marshall(response);
      Signer.signObject(signature);

      Response signedResponse = (Response) unmarshallerFactory.getUnmarshaller(
          response.getDOM()).unmarshall(response.getDOM());

      Marshaller m = marshallerFactory.getMarshaller(signedResponse);
      Element generatedDOM = m.marshall(response, parser.newDocument());
      BASE64Encoder be = new BASE64Encoder();
      String xml = XMLHelper.nodeToString(generatedDOM);
      return be.encode(xml.getBytes());
    } catch (Throwable t) {
      t.printStackTrace();
    }
    return null;
  }

  private static BasicCredential readCredentials(String publicKeyFilename,
      String privateKeyFilename) throws IOException, NoSuchAlgorithmException,
      InvalidKeySpecException, CertificateException {
    KeyFactory kf = KeyFactory.getInstance("RSA");
    InputStream fl = fullStream(privateKeyFilename);
    byte[] key = new byte[fl.available()];
    fl.read(key, 0, fl.available());
    fl.close();
    PKCS8EncodedKeySpec keysp = new PKCS8EncodedKeySpec(key);
    PrivateKey ff = kf.generatePrivate(keysp);
    CertificateFactory cf = CertificateFactory.getInstance("X.509");
    InputStream certstream = fullStream(publicKeyFilename);
    Certificate cert = cf.generateCertificate(certstream);
    return SecurityHelper.getSimpleCredential((X509Certificate) cert, ff);
  }

  private static InputStream fullStream(String fname) throws IOException {
    FileInputStream fis = new FileInputStream(fname);
    DataInputStream dis = new DataInputStream(fis);
    byte[] bytes = new byte[dis.available()];
    dis.readFully(bytes);
    ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
    return bais;
  }

  private XMLObject buildXMLObject(QName objectQName) throws RuntimeException {
    XMLObjectBuilder builder = Configuration.getBuilderFactory().getBuilder(
        objectQName);
    if (builder == null) {
      throw new RuntimeException("Unable to retrieve builder for object QName "
          + objectQName);
    }
    return builder.buildObject(objectQName.getNamespaceURI(), objectQName
        .getLocalPart(), objectQName.getPrefix());
  }

  public static void main(String[] args) {
    try {
      SAMLDemo demo = new SAMLDemo();
      System.out.println(demo.createSAMLResponse());
    } catch (Throwable t) {
      t.printStackTrace();
    }
  }

}